﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MyClock
{
    class Config
    {
        /// <summary>
        /// <Ver 1.17.3d></Ver>
        /// INIファイルのフルパスを取得する
        /// </summary>
        /// <returns>INIファイルのフルパス</returns>
        private string GetIniFilePath()
        {
            string fullpath = "";
            bool err;
            // MyClock.exe のパスを取得し、MyClock.ini のフルパスを得る
            fullpath = System.IO.Path.GetDirectoryName(Application.ExecutablePath) + "\\MyClock.ini";
            // MyClock.ini の存在を確認する
            err = !System.IO.File.Exists(fullpath);
            // MyClock.ini が存在しない時は空の MyClock.ini を作る
            if (err)
            {
                using (System.IO.FileStream hStream = System.IO.File.Create(fullpath))
                {
                    // 作成時に返される FileStream を利用して閉じる
                    if (hStream != null)
                    {
                        hStream.Close();
                    }
                }
            }
            return fullpath;
        }

        /// <summary>
        /// <Ver 1.17.6></Ver>
        /// 外観情報を削除する
        /// </summary>
        /// <param name="index">カタログインデックス</param>
        public void RemoveRegInfo(int index)
        {
            string inifile = GetIniFilePath();
            IniFile.SetIniString("Index " + index.ToString(), null, null, inifile);
        }

        /// <summary>
        /// <Ver 1.17.6></Ver>
        /// 指定したデフォルトの外観情報を保存する
        /// </summary>
        /// <param name="index">カタログインデックス</param>
        public void SetRegInfo(int index)
        {
            string inifile = GetIniFilePath();

            IniFile.SetIniString("Index Info", "Number of Index" ,DispReg.s_MaxIndex, inifile);
            IniFile.SetIniString("Index " + index.ToString(), "背景色", DispReg.s_BackColor, inifile);
            IniFile.SetIniString("Index " + index.ToString(), "フォント.Name", DispReg.s_Font_Name, inifile);
            IniFile.SetIniString("Index " + index.ToString(), "フォント.Size", DispReg.s_Font_Size, inifile);
            IniFile.SetIniString("Index " + index.ToString(), "フォント.Style", DispReg.s_Font_Style, inifile);
            IniFile.SetIniString("Index " + index.ToString(), "フォント.Unit", DispReg.s_Font_Unit, inifile);
            IniFile.SetIniString("Index " + index.ToString(), "フォント.GdiCharSet", DispReg.s_Font_GdiCharSet, inifile);
            IniFile.SetIniString("Index " + index.ToString(), "フォント.GdiVertialFont", DispReg.s_Font_GdiVertialFont, inifile);
            IniFile.SetIniString("Index " + index.ToString(), "フォント.Color", DispReg.s_FontColor, inifile);
            IniFile.SetIniString("Index " + index.ToString(), "サイズ.Width", DispReg.s_Size_Width, inifile);
            IniFile.SetIniString("Index " + index.ToString(), "サイズ.Height", DispReg.s_Size_Height, inifile);
            IniFile.SetIniString("Index " + index.ToString(), "不透明度", DispReg.s_Opacity, inifile);
            IniFile.SetIniString("Index " + index.ToString(), "枠のスタイル", DispReg.s_Border, inifile);
        }

        /// <summary>
        /// <Ver 1.17.6></Ver>
        /// 最大インデックス番号を取得する
        /// </summary>
        /// <returns>
        /// - 正常終了時: max_index (>0)
        /// - 異常時: -1
        /// </returns>
        public int GetRegIndexCount()
        {
            string inifile = GetIniFilePath(), s = "";
            int max_index;
            
            s = IniFile.GetIniString("Index Info", "Number of Index", inifile);
            if (int.TryParse(s, out max_index))
            {
                if (max_index < 0) { max_index = -1; }
            }
            else
            {
                if (s == "") { max_index = 0; }
                else { max_index = -1; }
            }

            return max_index;
        }

        /// <summary>
        /// <Ver 1.17.6></Ver>
        /// 最大インデックス番号を保存する
        /// </summary>
        /// <param name="index">カタログインデックス</param>
        public void SetRegIndexCount(int max_index)
        {
            string inifile = GetIniFilePath();
            IniFile.SetIniString("Index Info", "Number of Index", max_index.ToString(), inifile);
        }

        /// <summary>
        /// <Ver 1.17.6></Ver>
        /// 外観情報を取得する
        /// </summary>
        /// <param name="index">カタログインデックス</param>
        /// <returns>
        /// - 正常終了時: true
        /// - 異常時: false
        /// </returns>
        public bool GetRegInfo(int index)
        {
            string inifile = GetIniFilePath(), s = "";
            bool err = false;

            s = IniFile.GetIniString("Index " + index.ToString(), "背景色", inifile);
            if (s != "") { DispReg.s_BackColor = s; }
            else { err = true; }

            s = IniFile.GetIniString("Index " + index.ToString(), "フォント.Name", inifile);
            if (s != "")
            {
                DispReg.s_Font_Name = s;
                DispReg.s_Font_Size = IniFile.GetIniString("Index " + index.ToString(), "フォント.Size", inifile);
                DispReg.s_Font_Style = IniFile.GetIniString("Index " + index.ToString(), "フォント.Style", inifile);
                DispReg.s_Font_Unit = IniFile.GetIniString("Index " + index.ToString(), "フォント.Unit", inifile);
                DispReg.s_Font_GdiCharSet = IniFile.GetIniString("Index " + index.ToString(), "フォント.GdiCharSet", inifile);
                DispReg.s_Font_GdiVertialFont = IniFile.GetIniString("Index " + index.ToString(), "フォント.GdiVertialFont", inifile);
            }
            else { err = true; }

            s = IniFile.GetIniString("Index " + index.ToString(), "フォント.Color", inifile);
            if (s != "") { DispReg.s_FontColor = s; }
            else { err = true; }

            s = IniFile.GetIniString("Index " + index.ToString(), "サイズ.Width", inifile);
            if (s != "") { DispReg.s_Size_Width = s; }
            else { err = true; }

            s = IniFile.GetIniString("Index " + index.ToString(), "サイズ.Height", inifile);
            if (s != "") { DispReg.s_Size_Height = s; }
            else { err = true; }

            s = IniFile.GetIniString("Index " + index.ToString(), "不透明度", inifile);
            if (s != "") { DispReg.s_Opacity = s; }
            else { err = true; }

            s = IniFile.GetIniString("Index " + index.ToString(), "枠のスタイル", inifile);
            if (s != "") { DispReg.s_Border = s; }
            else { err = true; }

            return !err;
        }

        /// <summary>
        /// <Ver 1.17.6></Ver>
        /// 指定したデフォルトの外観情報を取得する
        /// </summary>
        /// <param name="inormDisp">
        /// - true: 通常の外観
        /// - false: アラーム設定時の外観</param>
        /// <returns>
        /// - 正常終了時: true
        /// - 異常時: false
        /// </returns>
        public bool GetDefaultRegInfo(bool normDisp)
        {
            string inifile = GetIniFilePath(), s = "", field = "";
            bool err = false;

            if (normDisp) { field = "通常の外観"; }
            else { field = "アラーム設定時の外観"; }
            
            s = IniFile.GetIniString(field, "背景色", inifile);
            if (s != "") { DispReg.s_BackColor = s; }
            else { err = true; }

            s = IniFile.GetIniString(field, "フォント.Name", inifile);
            if (s != "")
            {
                DispReg.s_Font_Name = s;
                DispReg.s_Font_Size = IniFile.GetIniString(field, "フォント.Size", inifile);
                DispReg.s_Font_Style = IniFile.GetIniString(field, "フォント.Style", inifile);
                DispReg.s_Font_Unit = IniFile.GetIniString(field, "フォント.Unit", inifile);
                DispReg.s_Font_GdiCharSet = IniFile.GetIniString(field, "フォント.GdiCharSet", inifile);
                DispReg.s_Font_GdiVertialFont = IniFile.GetIniString(field, "フォント.GdiVertialFont", inifile);
            }
            else { err = true; }

            s = IniFile.GetIniString(field, "フォント.Color", inifile);
            if (s != "") { DispReg.s_FontColor = s; }
            else { err = true; }

            s = IniFile.GetIniString(field, "サイズ.Width", inifile);
            if (s != "") { DispReg.s_Size_Width = s; }
            else { err = true; }

            s = IniFile.GetIniString(field, "サイズ.Height", inifile);
            if (s != "") { DispReg.s_Size_Height = s; }
            else { err = true; }

            s = IniFile.GetIniString(field, "不透明度", inifile);
            if (s != "") { DispReg.s_Opacity = s; }
            else { err = true; }

            s = IniFile.GetIniString(field, "枠のスタイル", inifile);
            if (s != "") { DispReg.s_Border = s; }
            else { err = true; }

            return !err;
        }

        /// <summary>
        /// <Ver 1.18.0></Ver>
        /// INIファイルのサイズを返す
        /// </summary>
        /// <returns></returns>
        public long GetInifileSize()
        {
            string inifile = GetIniFilePath();

            System.IO.FileInfo fileinfo = new System.IO.FileInfo(inifile);
            fileinfo.Refresh();
            //ファイルのサイズを取得
            long filesize = fileinfo.Length;

            return filesize;
        }
        
        /// <summary>
        /// <Ver 1.17.3d></Ver>
        /// INIファイルに書き込む
        /// </summary>
        public void SetIniFile()
        {
            string inifile = GetIniFilePath();

            IniFile.SetIniString("＜＜注意＞＞", "***", "以下の設定は直接編集しないこと！***", inifile);
            IniFile.SetIniString("Assembly Info", "Title", About.getAssemblyTitle(), inifile);
            IniFile.SetIniString("Assembly Info", "Product", About.getAssemblyProduct(), inifile);
            IniFile.SetIniString("Assembly Info", "Verson", About.getAssemblyVersion(), inifile);
            IniFile.SetIniString("Application", "Alarm Time", Display.m_Alarm, inifile);
            IniFile.SetIniString("Application", "Alarm Canceled", Display.m_cancelAlarm ? "true" : "false", inifile);
            IniFile.SetIniString("Application", "Position.X", Display.m_main_pos.X.ToString(), inifile);
            IniFile.SetIniString("Application", "Position.Y", Display.m_main_pos.Y.ToString(), inifile);
            IniFile.SetIniString("Application", "isSize_normal", Display.m_size_normal ? "true" : "false", inifile);
            IniFile.SetIniString("Application", "Norm_alarm", Display.m_Norm_alarm.ToString(), inifile);
            IniFile.SetIniString("Application", "Icon_alarm", Display.m_Icon_alarm.ToString(), inifile);
            IniFile.SetIniString("Application", "doesShow_about", Display.m_show_about ? "true" : "false", inifile);
            IniFile.SetIniString("Application", "isStick", Display.m_stickDisp ? "true" : "false", inifile);
            IniFile.SetIniString("Application", "doesStartup", Display.m_startup ? "true" : "false", inifile);
            IniFile.SetIniString("通常の外観", "背景色", Display.m_NormBackColor, inifile);
            IniFile.SetIniString("アラーム設定時の外観", "背景色", Display.m_SetAlarmBackColor, inifile);
            IniFile.SetIniString("通常の外観", "フォント.Name", Display.m_NormFont.Name.ToString(), inifile);
            IniFile.SetIniString("通常の外観", "フォント.Size", Display.m_NormFont.Size.ToString(), inifile);
            IniFile.SetIniString("通常の外観", "フォント.Style", Display.m_NormFont.Style.ToString(), inifile);
            IniFile.SetIniString("通常の外観", "フォント.Unit", Display.m_NormFont.Unit.ToString(), inifile);
            IniFile.SetIniString("通常の外観", "フォント.GdiCharSet", Display.m_NormFont.GdiCharSet.ToString(), inifile);
            IniFile.SetIniString("通常の外観", "フォント.GdiVertialFont", Display.m_NormFont.GdiVerticalFont.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "フォント.Name", Display.m_SetAlarmFont.Name.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "フォント.Size", Display.m_SetAlarmFont.Size.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "フォント.Style", Display.m_SetAlarmFont.Style.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "フォント.Unit", Display.m_SetAlarmFont.Unit.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "フォント.GdiCharSet", Display.m_SetAlarmFont.GdiCharSet.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "フォント.GdiVertialFont", Display.m_SetAlarmFont.GdiVerticalFont.ToString(), inifile);

            IniFile.SetIniString("通常の外観", "フォント.Color", Display.m_NormFontColor, inifile);
            IniFile.SetIniString("アラーム設定時の外観", "フォント.Color", Display.m_SetAlarmFontColor, inifile);
            IniFile.SetIniString("通常の外観", "サイズ.Width", Display.m_DispNormSize.Width.ToString(), inifile);
            IniFile.SetIniString("通常の外観", "サイズ.Height", Display.m_DispNormSize.Height.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "サイズ.Width", Display.m_DispSetAlarmSize.Width.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "サイズ.Height", Display.m_DispSetAlarmSize.Height.ToString(), inifile);
            IniFile.SetIniString("通常の外観", "不透明度", Display.m_NormOpacity.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "不透明度", Display.m_SetAlarmOpacity.ToString(), inifile);
            IniFile.SetIniString("通常の外観", "枠のスタイル", Display.m_NormBorder.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "枠のスタイル", Display.m_SetAlarmBorder.ToString(), inifile);
        }

        /// <summary>
        /// <Ver 1.17.3d></Ver>
        /// INIファイルから読み出す
        /// </summary>
        public void GetIniFile()
        {
            string inifile = GetIniFilePath(), s = "";

            s = IniFile.GetIniString("Application", "Alarm Time", inifile);
            if (s != "") { Display.m_Alarm = s; }
            else { Display.m_Alarm = "00:00:00"; }

            s = IniFile.GetIniString("Application", "Alarm Canceled", inifile);
            if (s == "true") { Display.m_cancelAlarm = true; }
            else { Display.m_cancelAlarm = false; }

            s = IniFile.GetIniString("Application", "Position.X", inifile);
            if (s != "") { Display.m_main_pos.X = int.Parse(s); }
            else { Display.m_main_pos.X = 100; }

            s = IniFile.GetIniString("Application", "Position.Y", inifile);
            if (s != "") { Display.m_main_pos.Y = int.Parse(s); }
            else { Display.m_main_pos.Y = 70; }

            s = IniFile.GetIniString("Application", "isSize_normal", inifile);
            if (s == "false") { Display.m_size_normal = false; }
            else { Display.m_size_normal = true; }

            s = IniFile.GetIniString("Application", "Norm_alarm", inifile);
            if (s != "") { Display.m_Norm_alarm = int.Parse(s); }
            else { Display.m_Norm_alarm = 1; }

            s = IniFile.GetIniString("Application", "Icon_alarm", inifile);
            if (s != "") { Display.m_Icon_alarm = int.Parse(s); }
            else { Display.m_Icon_alarm = 2; }

            s = IniFile.GetIniString("Application", "doesShow_about", inifile);
            if (s == "false") { Display.m_show_about = false; }
            else { Display.m_show_about = true; }

            s = IniFile.GetIniString("Application", "isStick", inifile);
            if (s == "true") { Display.m_stickDisp = true; }
            else { Display.m_stickDisp = false; }

            s = IniFile.GetIniString("Application", "doesStartup", inifile);
            if (s == "false") { Display.m_startup = false; }
            else { Display.m_startup = true; }

            s = IniFile.GetIniString("通常の外観", "背景色", inifile);
            if (s != "") { Display.m_NormBackColor = s; }
            else { Display.m_NormBackColor = "Info"; }

            s = IniFile.GetIniString("アラーム設定時の外観", "背景色", inifile);
            if (s != "") { Display.m_SetAlarmBackColor = s; }
            else { Display.m_SetAlarmBackColor = "ActiveCaption"; }

            s = IniFile.GetIniString("通常の外観", "フォント.Name", inifile);
            string name = s;
            if (name == "") { name = "MS UI Gothic"; }
            string size = IniFile.GetIniString("通常の外観", "フォント.Size", inifile);
            if (size == "") { size = "36F"; }
            string style = IniFile.GetIniString("通常の外観", "フォント.Style", inifile);
            if (style == "") { style = "Regular"; }
            string unit = IniFile.GetIniString("通常の外観", "フォント.Unit", inifile);
            if (unit == "") { unit = "Point"; }
            string gcs = IniFile.GetIniString("通常の外観", "フォント.GdiCharSet", inifile);
            if (gcs == "") { gcs = "1"; } 
            string gvf = IniFile.GetIniString("通常の外観", "フォント.GdiVertialFont", inifile);
            if (gvf == "") { gvf = "False"; }
            if (s != "")
            {
                Display.m_NormFont = new Font(
                    name,
                    float.Parse(size),
                    (FontStyle)Enum.Parse(typeof(FontStyle), style),
                    (GraphicsUnit)Enum.Parse(typeof(GraphicsUnit), unit),
                    byte.Parse(gcs),
                    bool.Parse(gvf));
            }
            else
            {
                Display.m_NormFont = new Font("MS UI Gothic", 36F, FontStyle.Regular, GraphicsUnit.Point, ((byte)(128))); 
            }

            s = IniFile.GetIniString("アラーム設定時の外観", "フォント.Name", inifile);
            name = s;
            if (name == "") { name = "MS UI Gothic"; }
            size = IniFile.GetIniString("アラーム設定時の外観", "フォント.Size", inifile);
            if (size == "") { size = "36F"; }
            style = IniFile.GetIniString("アラーム設定時の外観", "フォント.Style", inifile);
            if (style == "") { style = "Regular"; }
            unit = IniFile.GetIniString("アラーム設定時の外観", "フォント.Unit", inifile);
            if (unit == "") { unit = "Point"; }
            gcs = IniFile.GetIniString("アラーム設定時の外観", "フォント.GdiCharSet", inifile);
            if (gcs == "") { gcs = "1"; }
            gvf = IniFile.GetIniString("アラーム設定時の外観", "フォント.GdiVertialFont", inifile);
            if (gvf == "") { gvf = "False"; }
            if (s != "")
            {
                Display.m_SetAlarmFont = new Font(
                    name,
                    float.Parse(size),
                    (FontStyle)Enum.Parse(typeof(FontStyle), style),
                    (GraphicsUnit)Enum.Parse(typeof(GraphicsUnit), unit),
                    byte.Parse(gcs),
                    bool.Parse(gvf));
            }
            else
            {
                Display.m_SetAlarmFont = new Font("MS UI Gothic", 36F, FontStyle.Regular, GraphicsUnit.Point, ((byte)(128)));
            }

            s = IniFile.GetIniString("通常の外観", "フォント.Color", inifile);
            if (s != "") { Display.m_NormFontColor = s; }
            else { Display.m_NormFontColor = "ControlText"; }

            s = IniFile.GetIniString("アラーム設定時の外観", "フォント.Color", inifile);
            if (s != "") { Display.m_SetAlarmFontColor = s; }
            else { Display.m_SetAlarmFontColor = "ControlText"; }

            s = IniFile.GetIniString("通常の外観", "サイズ.Width", inifile);
            if (s != "") { Display.m_DispNormSize.Width = int.Parse(s); }
            else { Display.m_DispNormSize.Width = 191; }

            s = IniFile.GetIniString("通常の外観", "サイズ.Height", inifile);
            if (s != "") { Display.m_DispNormSize.Height = int.Parse(s); }
            else { Display.m_DispNormSize.Height = 46; }

            s = IniFile.GetIniString("アラーム設定時の外観", "サイズ.Width", inifile);
            if (s != "") { Display.m_DispSetAlarmSize.Width = int.Parse(s); }
            else { Display.m_DispSetAlarmSize.Width = 191; }

            s = IniFile.GetIniString("アラーム設定時の外観", "サイズ.Height", inifile);
            if (s != "") { Display.m_DispSetAlarmSize.Height = int.Parse(s); }
            else { Display.m_DispSetAlarmSize.Height = 46; }

            s = IniFile.GetIniString("通常の外観", "不透明度", inifile);
            if (s != "") { Display.m_NormOpacity = double.Parse(s); }
            else { Display.m_NormOpacity = 0.5D; }

            s = IniFile.GetIniString("アラーム設定時の外観", "不透明度", inifile);
            if (s != "") { Display.m_SetAlarmOpacity = double.Parse(s); }
            else { Display.m_SetAlarmOpacity = 0.5D; }

            s = IniFile.GetIniString("通常の外観", "枠のスタイル", inifile);
            if (s != "") { Display.m_NormBorder = s; }
            else { Display.m_NormBorder = "None"; }

            s = IniFile.GetIniString("アラーム設定時の外観", "枠のスタイル", inifile);
            if (s != "") { Display.m_SetAlarmBorder = s; }
            else { Display.m_SetAlarmBorder = "None"; }
        }
    }
}
